"""
    Builder Web Services
"""

from .admin.bootstrap import bootstrap
from .admin import urls
from .admin import json
from .admin import view
from .admin import create_admin
# from .admin.cmd import create_app
# from .admin.cmd import create_project


__version__ = "0.0.4a0"
__author__ = "Jose Angel Delgado"
__author_email__ = "esojangel@gmail.com"


__all__ = [
    "bootstrap",
    "urls",
    "json",
    "view",
    "create_admin",
]



template = """

from app import create_app
from boa.admin.bootstrap import bootstrap

config_file = os.getenv('APP_CONFIG_FILE', "app.config.py")


bootstrap(app, admin, ["config", "api"])
admin = create_app(app)


from app import create_app, admin
from boa.admin.bootstrap import bootstrap


config_file = os.getenv('APP_CONFIG_FILE', "app.config.py")
app = create_app()
bootstrap(app, admin, ["config", "view", "routes"])

if __name__ == '__main__':    
    app.run(host=app.config["HOST"], port=app.config["PORT"])

"""



template='''
from boa.admin.view import BaseFrontendView, BaseView
from boa.admin.urls import expose_json
from app import admin


@admin.expose("%(app)s/frontview")
class FrontView(BaseFrontendView):
    """
        Vista para interfaces registradas en el nodo principal (MAPP) 
    """
    def setup(self):
        self.name = "ms_%(app)s"
        """
            Icono que se usará en MAPP
        """
        self.ico = "fa-gear"
        """
            Ruta de esta vista en MAPP
        """
        self.endpoint_frontend = "-/ms/%(app)s"
        """
            Ruta para la api de la vista
        """
        self.endpoint_api = "%(app)s/frontview"

    @expose_json("/js")
    def js(self):
        """
            colocar los js
        """
        return []

    @expose_json("/css")
    def css(self):
        """
            colocar los ccs
        """
        return []
    
    @expose_json("/ctx")
    def context(self):
        """
            colocar los ctx
        """
        return {}
    
    @expose_json(url='/test', methods=('GET',))
    def get_test(self):
        return { "success" : True}


@admin.expose("%(app)s/apiview")
class ApiView(BaseView):
    """
        Vista para api
    """
    
    @expose_json(url='/test', methods=('GET',))
    def get_test(self):
        return { "success" : True}

'''



template = """
DB_NAME=ms-name
DB_ENGINE=mysql+pymysql
DB_USER=ms-user
DB_PASSWORD=ms-pass
DB_HOST=127.0.0.1
DB_PORT=3306

JWT_SECRET_KEY=%(secret)s

REGISTER=http://ip_node:5555/ms/register
MS_HOST=https://ip_ms:5000

HOST=localhost
PORT=5000
DEBUG=false
"""



import logging
from .base import create_app
logger = logging.getLogger("Boot Admin")


if __name__ == "__main__":
    logging.basicConfig(
        level=logging.INFO,
        format='[%(levelname).1s %(asctime).19s] %(message)s',
        datefmt='%y-%m-%d %H:%M:%S'
    )

    create_app()



import argparse
import logging
import os
import re
from os import path
from .. import templates as tpls
from uuid import uuid1
import flask_migrate
from flask import current_app
import subprocess

logger = logging.getLogger("")


def create_files_from_tlp(dir_tpl, tpl, dir_target, target):
    with open(path.join(dir_tpl, tpl), "r") as f_tpl:
        tpl_body = f_tpl.read()
        os.makedirs(dir_target, exist_ok=1)
        with open(path.join(dir_target, target), "w+") as f:
            f.write(tpl_body)

def create_files(tpl_body, dir_target, target):
    os.makedirs(dir_target, exist_ok=1)
    if os.path.exists(path.join(dir_target, target)):
        return 
    with open(path.join(dir_target, target), "w+") as f:
        f.write(tpl_body)


def create_project(project=os.getcwd(), name="name"):

    # logger.info("%s" % args.name)
    if project == ".":
        target = os.getcwd()
    else:
        if not project.startswith("/"):
            target = path.join(os.getcwd(), project)
        target = project

    target_app = path.join(target, "app")
    # target_app_main = path.join(target, f"app/{name}")
    logger.info("target : %s" % target)
    create_files(tpls.bin, target, "bin")
    # create_files(tpls.web, target, "web.py")
    # create_files(tpls.cmd, target, "cmd.py")
    # create_files(tpls.migrate, target, "migrate.py")
    create_files(tpls.entrypoint, target, "entrypoint.py")
    create_files(tpls.requirements_txt, target, "requirements.txt")
    create_files(tpls.readme, target, "README.md")
    create_files(tpls.setting, target_app, "__init__.py")
    create_files(tpls.config % dict(app=name), target_app, "config.py")
    create_files(tpls.env % dict(secret=uuid1()), target, ".env")
    
    create_app(target_app, name, ask_confirm=False)
    # create_files(tpls.app_config % dict(app=name), target_app_main, "config.py")
    # create_files(tpls.app_routes % dict(app=re.sub(r"[^\w]", "", name)), target_app_main, "routes.py")
    # create_files(tpls.app_cmd % dict(app=re.sub(r"[^\w]", "", name.lower())), target_app_main, "cmd.py")
    # create_files(tpls.app_migrations % dict(app=re.sub(r"[^\w]", "", name.lower())), target_app_main, "migrations.py")
    # create_files(tpls.app_view  % dict(app=name), target_app_main, "view.py")
    # create_files(tpls.app_models % dict(app=name), target_app_main, "models.py")
    # with current_app.app_context():
        # flask_migrate.init()
    
    # os.environ["FLASK_APP"] = target_app
    # subprocess.run(
    #     "flask db init".split(" "), stdout=subprocess.PIPE
    # )
    print("""\
Run
$ boa db init
    """)


def create_app(folder, name=None, ask_confirm=True):
    
    logger.info("%s" % name)
    name_app = path.basename(name)
    target_app_main = path.join(folder, name)
    create_files(tpls.app_view % dict(app=name_app), target_app_main, "view.py")
    
    def confirm(txt):
        if ask_confirm:
           return input(txt).lower().strip() in ["y", "s", "si", "yes"]
        return True 

    if confirm("create config.py?[y/n]: "):
        create_files(tpls.app_config % dict(app=name_app), target_app_main, "config.py")
    if confirm("create routes.py?[y/n]: "):
        create_files(tpls.app_routes % dict(app=re.sub(r"[^\w]", "", name_app)), target_app_main, "routes.py")
    if confirm("create models.py?[y/n]: "):
        create_files(tpls.app_models % dict(app=name_app), target_app_main, "models.py")
    if confirm("create cmd.py?[y/n]: "):
        create_files(tpls.app_cmd % dict(app=re.sub(r"[^\w]", "", name_app.lower())), target_app_main, "cmd.py")
    # create_files(tpls.app_migrations % dict(app=re.sub(r"[^\w]", "", name_app.lower())), target_app_main, "migrations.py")


if __name__ == "__main__":
    logging.basicConfig(
        level=logging.INFO,
        format='[%(levelname).1s %(asctime).19s] %(message)s',
        datefmt='%y-%m-%d %H:%M:%S'
    )

    create_project()



# generado automaticamente
import sys
import os
import logging

logging.basicConfig(
    stream=sys.stdout,
    level=logging.INFO,
    format='[%(levelname).1s %(asctime).19s] %(message)s',
    datefmt='%y-%m-%d %H:%M:%S'
)
logger = logging.getLogger(__name__)


from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
# from flask_script import Manager, Command
from flask import Flask, Blueprint, url_for, jsonify
from flask import current_app as app
from flask.cli import AppGroup
from boa.admin.json import JSONEncoderCurstom
from boa.admin import create_admin
from boa.admin.bootstrap import bootstrap

admin = create_admin()
db = SQLAlchemy()
api = Blueprint('api', __name__)
cmd = AppGroup("cmd", short_help="Commands of apps")
migrate = Migrate()

config_file = os.getenv('APP_CONFIG_FILE', "config.py")


def create_app(config=config_file):
    _app = Flask(__name__, static_folder="../static")
    _app.config.from_pyfile(config)
    _app.json_encoder = JSONEncoderCurstom
    
    exts = [
        admin, db
    ]
    migrate.init_app(_app, db)

    for ext in exts:
        if callable(getattr(ext, "init_app", None)):
            ext.init_app(_app)

    @_app.before_first_request
    def regiter_frontends():
        admin.bootstrap_frontends()
    
    
    with _app.app_context():
        _app.register_blueprint(api)
        _app.cli.add_command(cmd)
        bootstrap(_app, admin, ["config", "view", "routes", "models"])
        if "cmd" in sys.argv:
            bootstrap(_app, None, ["cmd"])

    return _app




from .web import template as web
from .env import template as env
from .cmd import template as cmd
from .migrate import template as migrate
from .config import template as config
from .setting import template as setting
from .requirements_txt import template as requirements_txt
from .readme import template as readme
from .bin import template as bin
from .entrypoint import template as entrypoint
from .app.routes import template as app_routes
from .app.cmd import template as app_cmd
from .app.migrations import template as app_migrations
from .app.config import template as app_config
from .app.view import template as app_view
from .app.models import template as app_models



import flask_admin
from .urls import expose
from flask import jsonify

class BaseView(flask_admin.base.BaseView):
    enable = True
    allow_access = True

    def _handle_view(self, name, **kwargs):
        if self.enable:
            return super()._handle_view(name, **kwargs)
        else:
            return 'Failure!'
    
    @expose("/")
    def index(self):
        return ""
    
    @expose("/urls")
    def method_get_urls(self):
        return jsonify(self._urls)


class BaseFrontendView(BaseView):
    """
        Vista para interfaces registradas en el nodo principal (MAPP) 
    """
    def __init__(self, *args, **kargs):
        super().__init__(*args, **kargs)
        self.name = "MS"
        """
            Icono que se usará en MAPP
        """
        self.ico = "fa-gear"
        """
            Ruta de esta vista en MAPP
        """
        self.endpoint_frontend = "ms1"
        """
            Ruta para la api de la vista
        """
        self.endpoint_api = None
        self.setup()

    def setup(self):
        pass

    def js(self):
        """
            colocar los js
        """
        return []

    def css(self):
        """
            colocar los ccs
        """
        return []
    
    def context(self):
        """
            colocar los ctx
        """
        return {}




from boa.admin.urls import json_render
from app import api


@api.route("/main")
@json_render
def get_main_api():
    return {
        "success": True, 
        "message": "Metodo de api para api/main"
    }




#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['boa', 'boa.admin', 'boa.cmd', 'boa.templates', 'boa.utils']

package_data = \
{'': ['*'], 'boa': ['plop-templates/*'], 'boa.templates': ['app/*']}

install_requires = \
['werkzeug==0.16.1',
 'flask==1.1.1',
 'click>=8.0.1',
 'flask-admin>=1.5.8',
 'Flask-SQLAlchemy>=2.4.1',
 'Flask-Migrate==2.5.2',
 'flask-babel>=0.12.2',
 'itsdangerous==2.0.1',
 'jinja2==3.0.1',
 'markupsafe==2.0.1',
 'requests>=2.26.0',
 'wtforms==2.3.3',
 'pytz==2021.1',
 'SQLAlchemy>=1.3.11',
 'python-dotenv>=0.18.0',
 'PyMySQL>=0.9.3']

entry_points = \
{'console_scripts': ['boa = boa.cmd:boa_cli', 'bws = boa.cmd:boa_cli']}

setup(name='bws-boa',
      version='0.0.4a0',
      description='Builder Web Services',
      author='Jose Angel Delgado',
      author_email='esojangel@gmail.com',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )



template="""
from app import db, migrate
from .models import ExampleModel

'''
    boa migrate %(app)s__create_table
'''

@migrate.command
def %(app)s__create_table():
    return ExampleModel.__table__.create(db.get_engine())

"""



# from .view import FrontView
# from .view import ApiView

def register_frontend_views(admin):
    '''
        Llamamos esta funcion para registrar en el MAPP NODE 
    '''
    pass
    # admin.register_frontend(FrontView, endpoint="main/frontview/")


def register_api_views(admin):
    '''
        Llamamos esta funcion para registrar solo las urls
    '''
    pass
    # admin.register_api(ApiView, endpoint="main/apiview/")


